@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -46,7 +46,6 @@ enum connman_element_type {
 	CONNMAN_ELEMENT_TYPE_DEVICE     = 3,
 	CONNMAN_ELEMENT_TYPE_NETWORK    = 4,
 	CONNMAN_ELEMENT_TYPE_SERVICE    = 5,
-	CONNMAN_ELEMENT_TYPE_PPP        = 6,
 	CONNMAN_ELEMENT_TYPE_IPV4       = 7,
 	CONNMAN_ELEMENT_TYPE_IPV6       = 8,
 	CONNMAN_ELEMENT_TYPE_DHCP       = 9,
@@ -64,9 +63,10 @@ enum connman_element_state {
 };
 
 enum connman_element_error {
-	CONNMAN_ELEMENT_ERROR_UNKNOWN     = 0,
-	CONNMAN_ELEMENT_ERROR_FAILED      = 1,
-	CONNMAN_ELEMENT_ERROR_DHCP_FAILED = 2,
+	CONNMAN_ELEMENT_ERROR_UNKNOWN        = 0,
+	CONNMAN_ELEMENT_ERROR_FAILED         = 1,
+	CONNMAN_ELEMENT_ERROR_DHCP_FAILED    = 2,
+	CONNMAN_ELEMENT_ERROR_CONNECT_FAILED = 3,
 };
 
 struct connman_driver;
@@ -80,9 +80,9 @@ struct connman_element {
 	enum connman_element_state state;
 	enum connman_element_error error;
 	gboolean enabled;
-	gboolean configuring;
 	gchar *devname;
 
+	GHashTable *children;
 	struct connman_element *parent;
 
 	struct connman_driver *driver;
@@ -106,6 +106,8 @@ struct connman_element {
 		gchar *network;
 		gchar *broadcast;
 		gchar *nameserver;
+		gchar *timeserver;
+		gchar *pac;
 	} ipv4;
 };
 
@@ -120,6 +122,10 @@ int connman_element_set_string(struct connman_element *element,
 					const char *key, const char *value);
 const char *connman_element_get_string(struct connman_element *element,
 							const char *key);
+int connman_element_set_bool(struct connman_element *element,
+				const char *key, connman_bool_t value);
+connman_bool_t connman_element_get_bool(struct connman_element *element,
+							const char *key);
 int connman_element_set_uint8(struct connman_element *element,
 				const char *key, connman_uint8_t value);
 connman_uint8_t connman_element_get_uint8(struct connman_element *element,
