@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -91,8 +91,6 @@ static const char *type2string(enum connman_network_type type)
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 		return "bluetooth";
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 		return "cellular";
 	}
 
@@ -239,7 +237,8 @@ static void unregister_interface(struct connman_element *element)
 
 	network->registered = FALSE;
 
-	emit_networks_signal(network->device);
+	if (network->device != NULL)
+		emit_networks_signal(network->device);
 
 	g_dbus_unregister_interface(connection, element->path,
 						CONNMAN_NETWORK_INTERFACE);
@@ -445,6 +444,19 @@ const char *connman_network_get_path(struct connman_network *network)
  */
 void connman_network_set_index(struct connman_network *network, int index)
 {
+	if (network->element.index < 0) {
+		struct connman_service *service;
+
+		/*
+		 * This is needed for plugins that havent set their ipconfig
+		 * layer yet, due to not being able to get a network index
+		 * prior to creating a service.
+		 */
+		service = __connman_service_lookup_from_network(network);
+		if (service != NULL)
+			__connman_service_create_ipconfig(service, index);
+	}
+
 	network->element.index = index;
 }
 
@@ -502,8 +514,6 @@ void connman_network_set_group(struct connman_network *network,
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		break;
@@ -555,8 +565,6 @@ connman_bool_t __connman_network_get_weakness(struct connman_network *network)
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		break;
 	case CONNMAN_NETWORK_TYPE_WIFI:
@@ -652,10 +660,35 @@ static void set_associate_error(struct connman_network *network)
 					CONNMAN_SERVICE_STATE_FAILURE);
 }
 
+static void set_configure_error(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	network->connecting = FALSE;
+
+	service = __connman_service_lookup_from_network(network);
+
+	__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_FAILURE);
+}
+
 void connman_network_set_method(struct connman_network *network,
 					enum connman_ipconfig_method method)
 {
+	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
+
 	network->element.ipv4.method = method;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return;
+
+	ipconfig = __connman_service_get_ipconfig(service);
+	if (ipconfig == NULL)
+		return;
+
+	connman_ipconfig_set_method(ipconfig, method);
 }
 
 void connman_network_set_error(struct connman_network *network,
@@ -671,14 +704,17 @@ void connman_network_set_error(struct connman_network *network,
 	case CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL:
 		set_associate_error(network);
 		break;
+	case CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL:
+		set_configure_error(network);
+		break;
 	}
 }
 
-static void set_connected_manual(struct connman_network *network)
+static void set_configuration(struct connman_network *network)
 {
 	struct connman_service *service;
 
-	DBG("");
+	DBG("network %p", network);
 
 	__connman_device_increase_connections(network->device);
 
@@ -687,6 +723,32 @@ static void set_connected_manual(struct connman_network *network)
 	connman_device_set_disconnected(network->device, FALSE);
 
 	service = __connman_service_lookup_from_network(network);
+	__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_CONFIGURATION);
+}
+
+static void set_connected_manual(struct connman_network *network)
+{
+	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
+	int err;
+
+	DBG("network %p", network);
+
+	service = __connman_service_lookup_from_network(network);
+
+	ipconfig = __connman_service_get_ipconfig(service);
+
+	set_configuration(network);
+
+	err = __connman_ipconfig_set_address(ipconfig);
+	if (err < 0) {
+		connman_network_set_error(network,
+			CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL);
+		return;
+	}
+
+	__connman_ipconfig_set_gateway(ipconfig, &network->element);
 
 	network->connecting = FALSE;
 
@@ -698,7 +760,6 @@ static void set_connected_manual(struct connman_network *network)
 static int set_connected_dhcp(struct connman_network *network)
 {
 	struct connman_element *element;
-	struct connman_service *service;
 	int error;
 
 	DBG("network %p", network);
@@ -706,9 +767,6 @@ static int set_connected_dhcp(struct connman_network *network)
 	if (network->protocol != CONNMAN_NETWORK_PROTOCOL_IP)
 		return -EINVAL;
 
-	service = __connman_service_lookup_from_network(network);
-
-	DBG("a");
 	element = connman_element_create(NULL);
 	if (element == NULL)
 		return -ENOMEM;
@@ -722,14 +780,7 @@ static int set_connected_dhcp(struct connman_network *network)
 		return error;
 	}
 
-	__connman_device_increase_connections(network->device);
-
-	__connman_device_set_network(network->device, network);
-
-	connman_device_set_disconnected(network->device, FALSE);
-
-	__connman_service_indicate_state(service,
-			CONNMAN_SERVICE_STATE_CONFIGURATION);
+	set_configuration(network);
 
 	return 0;
 }
@@ -737,11 +788,20 @@ static int set_connected_dhcp(struct connman_network *network)
 static gboolean set_connected(gpointer user_data)
 {
 	struct connman_network *network = user_data;
+	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
+	enum connman_ipconfig_method method;
 
-	DBG("network method %d", network->element.ipv4.method);
+	service = __connman_service_lookup_from_network(network);
+
+	ipconfig = __connman_service_get_ipconfig(service);
+
+	method = __connman_ipconfig_get_method(ipconfig);
+
+	DBG("method %d", method);
 
 	if (network->connected == TRUE) {
-		switch (network->element.ipv4.method) {
+		switch (method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
 			return FALSE;
@@ -872,8 +932,10 @@ int __connman_network_connect(struct connman_network *network)
 	if (err < 0) {
 		if (err == -EINPROGRESS)
 			connman_network_set_associating(network, TRUE);
-		else
+		else {
+			network->connecting = FALSE;
 			network->hidden = FALSE;
+		}
 
 		return err;
 	}
@@ -910,7 +972,7 @@ int __connman_network_disconnect(struct connman_network *network)
 
 	err = network->driver->disconnect(network);
 	if (err == 0) {
-		network->connected = FALSE;
+		connman_network_set_connected(network, FALSE);
 		set_connected(network);
 	}
 
@@ -1014,6 +1076,8 @@ int connman_network_set_roaming(struct connman_network *network,
 int connman_network_set_string(struct connman_network *network,
 					const char *key, const char *value)
 {
+	int err;
+
 	DBG("network %p key %s value %s", network, key, value);
 
 	if (g_strcmp0(key, "Name") == 0)
@@ -1057,7 +1121,17 @@ int connman_network_set_string(struct connman_network *network,
 		network->wifi.phase2_auth = g_strdup(value);
 	}
 
-	return connman_element_set_string(&network->element, key, value);
+	err = connman_element_set_string(&network->element, key, value);
+	if (err < 0)
+		return err;
+
+	if (network->driver == NULL)
+		return 0;
+
+	if (network->driver->setup)
+		return network->driver->setup(network, key);
+
+	return 0;
 }
 
 /**
@@ -1362,8 +1436,6 @@ static int network_probe(struct connman_element *element)
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		if (network->group != NULL)
@@ -1394,8 +1466,6 @@ static void network_remove(struct connman_element *element)
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 	case CONNMAN_NETWORK_TYPE_CELLULAR:
-	case CONNMAN_NETWORK_TYPE_MBM:
-	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		if (network->group != NULL) {
